# $Id: preferences.tcl,v 1.1 2003/11/25 10:42:45 davidw Exp $

# preferences.tcl --

# Automate the creation of a simple preferences dialog, with a
# callback for the 'save' functionality.

# Copyright 2003 David N. Welton <davidw@dedasys.com>

package require Tk

if { [lsearch $auto_path .] < 0 } {
    lappend auto_path .
}

package require gridinfo

package provide preferencesgui 0.1

namespace eval prefs {
    set id 0
    array set prefs {}
    set callback {}
}


# prefs::addwidget --
#
#	Add a widget to the preferences dialog.
#
# Arguments:
#	FIXME.
#
# Side Effects:
#	Set variable on 'save'.
#
# Results:
#	None.

proc prefs::addwidget {label type varname {default ""}} {
    variable id
    variable p
    variable var2widget
    variable prefs

    set pw $p.widgets

    labelframe $pw.f$id -text $label
    grid $pw.f$id -sticky new -ipadx 8 -ipady 4
    set datatype [lindex $type 0]
    switch -exact $datatype {
	file {
	    entry $pw.f$id.e -textvariable prefs::prefs($varname)
	    set prefs::prefs($varname) $default
	    button $pw.f$id.b -text "Browse..." -command \
		[list prefs::setfile $pw.f$id.e]
	    grid $pw.f$id.e $pw.f$id.b -sticky news
	    grid columnconfigure $pw.f$id 0 -weight 3
	}
    }

    incr id
}


# prefs::create --
#
#	Create the preferences dialog.
#
# Arguments:
#	userarr - the array to tie the dialog to.
#	callbackproc - procedure to call on save.
#
# Side Effects:
#	None.
#
# Results:
#	None.

proc prefs::create {callbackproc} {
    variable p
    variable prefs
    variable callback $callbackproc

    catch { destroy .prefs }
    toplevel .prefs
    set p .prefs

    wm title $p "Preferences"

    label $p.title -text "Preferences"

    frame $p.widgets

    button $p.save -text "Save Changes"  -command prefs::save
    button $p.cancel -text "Close" -command prefs::cancel

    grid $p.title -columnspan 2 -sticky new
    grid $p.widgets -columnspan 2 -sticky new
    grid columnconfigure $p.widgets 0 -weight 1
    grid $p.save $p.cancel -sticky s
    grid configure $p.save -sticky w
    grid configure $p.cancel -sticky e

    grid columnconfigure $p 0 -weight 1
    grid rowconfigure $p 1 -weight 1
    raise $p
}


# prefs::cancel --
#
#	Cancels/destroys preferencies dialog.  Changes are *not*
#	saved.
#
# Arguments:
#	None.
#
# Side Effects:
#	None.
#
# Results:
#	None.

proc prefs::cancel {} {
    variable p
    destroy $p
}


# prefs::save --
#
#	Saves the user supplied preferences by passing the new values
#	to the callback proc passed to 'create'.
#
# Arguments:
#	None.
#
# Side Effects:
#	None.
#
# Results:
#	None.

proc prefs::save {} {
    variable prefs
    variable callback
    $callback [array get prefs]
}

proc prefs::setfile {entry} {
    set types {
	{"Tcl Template" .tml}
	{"All files" *}
    }

    $entry delete 0 end
    $entry insert 0 [tk_getOpenFile -filetypes $types]
}