# progressbar.tcl -- implements a progress bar widget.

# $Id: progressbar.tcl,v 1.1 2004/09/21 10:05:06 davidw Exp $

package provide progressbar 0.1

package require msgcat

namespace eval progressbar {
    namespace import ::msgcat::*

    variable width
    set width 200
}

# progressbar::create --
#
#	Creates a progress bar widget.
#
# Arguments:
#	txt - the text to display regarding the task in progress.
#
# Side Effects:
#	Creates a new window, and destroys any previous progress bars
#
# Results:
#	None.

proc progressbar::create {txt} {
    variable cv
    variable width
    variable percentlabel
    catch [list destroy .progress]
    set p [toplevel .progress]
    wm title $p [mc "Progress"]
    set pf [labelframe $p.f -text $txt -padx 10 -pady 10]
    set cv [canvas $pf.cv -bg white -height 15 -width $width -relief sunken -bd 2]
    set percentlabel [label $pf.pl -width 6 -text "0%" -justify right]
    grid $pf -sticky news
    grid $cv $percentlabel -sticky ew
    grid columnconfigure .progress 0 -weight 1
    grid rowconfigure .progress 0 -weight 1
    set cvrect [$cv create rect 0 0 0 20 -fill navy -tag progbar]
    return $p
}

# progressbar::callback --
#
#	Callback that is passed to the function in progress.
#
# Arguments:
#	numer - numerator, or the number of the action that has been
#	completed (file size loaded, for instance), or files read.
#	denom - denominator - total actions to perform, or total that
#	equals completion, such as files to create or the total file
#	size to be read.
#
# Side Effects:
#	Updates the display.
#
# Results:
#	None.

proc progressbar::callback {numer denom} {
    variable cv
    variable width
    variable percentlabel
    set pct [expr {double($numer) / double($denom)}]
    set yext [expr {$width * $pct}]
    $percentlabel configure -text "[expr {int($pct * 100.0)}]%"
    $cv coords progbar 0 0 $yext 20
    update
}

# progressbar::done --
#
#	Callback indicating that the action in progress has been
#	completed.
#
# Arguments:
#	None.
#
# Side Effects:
#	Destroys the top level widget.
#
# Results:
#	None.

proc progressbar::done {} {
    catch { destroy .progress }
}
